package it.unibo.ai.rec.ui;


import it.unibo.ai.rec.model.HappenedEventSet;
import it.unibo.ai.rec.model.RecTrace;

import java.awt.Component;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.SwingUtilities;

import org.jfree.chart.axis.AxisState;
import org.jfree.chart.axis.DateTick;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTick;
import org.jfree.chart.axis.TickType;
import org.jfree.chart.axis.ValueTick;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.TextAnchor;

public class TraceNumberAxis extends NumberAxis implements HeightEvaluator {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8329828130639824937L;
	RecTrace trace;
	EventOutputter eventOutputter;
	
	public TraceNumberAxis(RecTrace trace, EventOutputter eventOutputter) {
		this.trace = trace;
		this.setVerticalTickLabels(true);
		this.eventOutputter = eventOutputter;
		setRange(0,trace.getHappenedEvents().size());
	}
	
	public List<?> refreshTicks(Graphics2D g2, AxisState state, Rectangle2D dataArea, RectangleEdge edge) {
		//super.refreshTicks(g2, state, dataArea, edge);
		List<ValueTick> result = new ArrayList<ValueTick>();
		int i = 0;
		for(HappenedEventSet set: trace.getHappenedEvents()) {
			Date date = eventOutputter.getTimestamp(set.getTime());
			result.add(
					new NumberTick(
							TickType.MAJOR,
							i,
							eventOutputter.toString(set), 
							TextAnchor.BASELINE_LEFT,
							TextAnchor.BASELINE_LEFT, Math.PI/2)
			);
			i++;
		}
		return result;		
		/*	
		for(int i = 0; i < trace.getEvents().size(); i++) {
			HappenedEvent event = trace.getEvents().get(i);
			result.add(
				new NumberTick(
					TickType.MAJOR,
					i,
					eventOutputter.toString(event), 
					TextAnchor.BASELINE_LEFT,
					TextAnchor.BASELINE_LEFT, 
					Math.PI/2
				)
			);
		}
		return result;
		*/
	}

	public int getHeight(Component c) {
		String biggestLabel="";
		for(HappenedEventSet set: trace.getHappenedEvents()) {
			String label = eventOutputter.toString(set);
			if(label.length() > biggestLabel.length())
				biggestLabel = label;
		}
		return SwingUtilities.computeStringWidth(c.getFontMetrics(getTickLabelFont()), biggestLabel+" ");
	}

}
